//  Fish Supper
//  Copyright (C) 2006 Matthew Clarke <matthew_larcke@lycos.co.uk>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.




#ifndef _ALBERT_H_
#define _ALBERT_H_




#include "sprite.h"
#include "enums.h"
#include "constants.h"
#include "log.h"
#include "playdisplay.h"
#include "crystal.h"
#include "SDL.h"
#include "fish.h"




namespace FS
{
    
class Albert : public Sprite
{
    private:
            
        Orientation my_orientation;
    
        char actions;
        
        bool airborne;
          
        int start_time; // for actions, but could there be more than one?
        int start_x;
        int start_y;

        // When Albert is being animated, this is the length of
        // time one image in the 'strip' remains on screen (in ms).
        // May need different ones for jumping, walking etc? Try to
        // make sure this isn't the case.
        int duration;             
        int num_frames; // i.e. in the walking strip
        int current_frame;
        ImageIDs current_strip;
        
        const Log * host_log;
        
        collision_box my_box;
        collision_box my_boxes[4];  // index using my_orientation (see above)
        
        void process_keypress(int t, KeyboardEvent ke);
        void process_animation(int t);
        
        void calc_rects();
        void calc_strip();
        void check_host_log();
        bool is_contained_by_log(const Log * l);
        
        SDL_Rect image_section_rect;
        
        void turn_on_action(int a) { actions |= a; }
        void turn_off_action(int a) { actions &= ~a; }
        int test_action(int a) const { return (actions & a); }
        
        // Albert landed in the water and splashing animation is over.
        bool splashed_zapped_out;
        
        bool left;
        
    public:
        Albert();
        ~Albert();
        
        void reset();
        void update(int t, KeyboardEvent ke, PlayDisplay * pd);
        
        //int get_current_frame() const { return current_frame; }
        //Orientation get_orientation() const { return my_orientation; }
        
        int get_collision_box_x1() const { return my_boxes[my_orientation].x1 + x; }
        int get_collision_box_x2() const { return my_boxes[my_orientation].x2 + x; }
        int get_collision_box_y1() const { return my_boxes[my_orientation].y1 + y; }
        int get_collision_box_y2() const { return my_boxes[my_orientation].y2 + y; }
                
        bool is_airborne() const { return airborne; }
        bool is_on_log() const { return (test_action(FLOATING) > 0); }
        int is_in_water_or_being_zapped() const; 
        
        bool check_new_log(const Log * l);
        bool check_crystal(const Crystal * c);
        bool check_fish(const Fish * f);
        
        int get_host_log_row() const { return host_log->get_row(); }
        
        void start_splash(int t);
        void start_zap(int t);
        
        void start_leave_with_fish(int t);
        void start_leave_in_a_huff(int t);
        
        bool is_splashed_zapped_out() const { return splashed_zapped_out; }
        
        bool has_left() const { return left; }
        
}; // class Albert

} // namespace FS




#endif
