//  Fish Supper
//  Copyright (C) 2006 Matthew Clarke <matthew_larcke@lycos.co.uk>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.




#include "crystal.h"
#include <iostream>

using std::cout;
using std::cerr;
using std::endl;




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::Crystal::Crystal()
{
    collision_pixel_x = CRYSTAL_COLLISION_PIXEL_X;
    collision_pixel_y = CRYSTAL_COLLISION_PIXEL_Y;
    
} // FS::Crystal::Crystal()

// ******************
// *** DESTRUCTOR ***
// ******************
FS::Crystal::~Crystal()
{
    // ?!
    
} // FS::Crystal::~Crystal()




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************

void FS::Crystal::update(int current_start_pixel, int pixels_moved, bool is_first_pass,
        PlayDisplay * pd)
{
    calc_x(current_start_pixel);
    calc_sprite_rect();
    
    // If this is the first pass, we don't need to bother updating
    // anything because the whole of the host log is going to be
    // updated anyway.
    if (is_first_pass)
    {
        update_visible = false;
        return;
    } // if
    
    // Initial values for dirty_rect.
    switch (my_direction)
    {
        case LEFT_TO_RIGHT:
            dirty_rect.x = x - pixels_moved;
            if (dirty_rect.x < 0)
            {
                dirty_rect.x += row_length_pixels;
            } // if
            
            dirty_rect.w = pixels_moved + width;
            
            break;
            
        case RIGHT_TO_LEFT:
            dirty_rect.x = x;
            
            dirty_rect.w = width + pixels_moved;
            
            break;
    } // switch
    
    update_visible = adjust_rect(&dirty_rect);
    
    if (sprite_visible)
    {
        pd->request_blit( ((ImageIDs) (current_color + IMAGE_ID_CRYSTAL_OFFSET)),
                NULL, &sprite_rect );
    } // if
    
    if (update_visible)
    {
        pd->request_update(dirty_rect);
    } // if
         
} // FS::Crystal::update()

// **************************************************

void FS::Crystal::init(int mx, int r, int yy, int w, int h, CrystalColor c, bool af, bool ab,
        int rlp, Direction d)
{
    map_x = mx + CRYSTAL_X_OFFSET;
    row = r;
    y = yy + CRYSTAL_Y_OFFSET;
    width = CRYSTAL_WIDTH;
    height = CRYSTAL_HEIGHT;
    original_color = c;
    current_color = original_color;
    at_front = af;
    at_back = ab;
    row_length_pixels = rlp;
    my_direction = d;
    
    active = true;
    
    sprite_rect.y = y;
    sprite_rect.w = width;
    sprite_rect.h = height;
    
    dirty_rect.y = y;
    dirty_rect.h = height;
    
} // FS::Crystal::init()
 
// **************************************************

void FS::Crystal::reset()
{
    current_color = original_color;
    active = true;
    
} // FS::Crystal::reset()

// **************************************************
// **************************************************
// **************************************************
