//  Fish Supper
//  Copyright (C) 2006 Matthew Clarke <matthew_larcke@lycos.co.uk>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.




#include "gamemanager.h"
#include <iostream>
#include <cstdlib>




using std::cout;
using std::cerr;
using std::endl;




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::GameManager::GameManager(SDL_Surface * s) : ts_display(s),
                                                gr_display(s),
                                                p_display(s)                                          
{
    screen = s;
    
    current_state = INTRO;
    
    p_model.init_sprite_manager(&p_display, &snd_engine);
    
} // FS::GameManager::GameManager()

// ******************
// *** DESTRUCTOR ***
// ******************
FS::GameManager::~GameManager()
{
    // does nothing yet...
    
} // FS::GameManager::~GameManager()




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************

void FS::GameManager::start_loop()
{
    SDL_Event event;
    my_key_event = NONE;
    loop_running = true;
    
    // Before we enter the loop, display the title screen
    ts_display.redraw();
        
    while (loop_running)
    {
        
        while ( SDL_PollEvent(&event) )
        {
            switch (event.type)
            {
                case SDL_KEYDOWN:
                case SDL_KEYUP:
                    analyse_event(&event);
                    break;
            } // switch
        } // while
        
        switch (current_state)
        {
            case INTRO:
                handle_selection( ts_display.update(my_key_event) );
                break;
                
            case GET_READY:
                gr_display.redraw(timer.get_time(), my_key_event);
                if (gr_display.is_finished())
                {
                    // start play display
                    current_state = PLAY;
                    p_model.new_level(timer.get_time());
                    p_display.reset();
                } // if
                break;
                
            case PLAY:
                if (my_key_event == PAUSE_KEY_PRESSED)
                {
                    timer.pause();
                } // if
                
                if (timer.is_running())
                {
                    switch ( p_model.update(timer.get_time(), my_key_event) )
                    {
                        case PR_OK:
                            p_display.redraw();
                            break;
                        case PR_LEVEL_COMPLETED:
                            current_state = GET_READY;
                            gr_display.start_draw( timer.get_time(), p_model.get_level() );
                            break;
                        case PR_GAME_OVER:
                        case PR_GAME_ABORTED:
                            current_state = INTRO;
                            ts_display.redraw();
                            break;
                    } // switch
                } // if
                break;
                
            case OPTIONS:
            case HI_SCORES:
                break;
                
        } // switch
        
        SDL_Delay(10);  // 100 fps
        //SDL_Delay(100);
        
	// IMPORTANT!!! Clear my_key_event!
	my_key_event = NONE;
        
    } // while

} // FS::GameManager::start_loop()

// **************************************************

void FS::GameManager::analyse_event(const SDL_Event * event)
{
    my_key_event = NONE;   // don't need this...
    
    if (event->type == SDL_KEYDOWN)
    {
        if (event->key.keysym.sym == SDLK_UP)
        {
            my_key_event = UP_KEY_PRESSED;
        }
        else if (event->key.keysym.sym == SDLK_RIGHT)
        {
            my_key_event = RIGHT_KEY_PRESSED;      
        }
        else if (event->key.keysym.sym == SDLK_DOWN)
        {
            my_key_event = DOWN_KEY_PRESSED;
        }
        else if (event->key.keysym.sym == SDLK_LEFT)
        {
            my_key_event = LEFT_KEY_PRESSED;
        }
        else if (event->key.keysym.sym == SDLK_SPACE)
        {
            my_key_event = JUMP_KEY_PRESSED;
        }
        else if (event->key.keysym.sym == SDLK_ESCAPE)
        {
            my_key_event = ESCAPE_PRESSED;
        }
        else if (event->key.keysym.sym == SDLK_p)
        {
            my_key_event = PAUSE_KEY_PRESSED;
        } // if ... else
    }
    else if (event->type == SDL_KEYUP)
    {
        if (event->key.keysym.sym == SDLK_UP)
        {
            my_key_event = UP_KEY_RELEASED;
        }
        else if (event->key.keysym.sym == SDLK_RIGHT)
        {
            my_key_event = RIGHT_KEY_RELEASED;
        }
        else if (event->key.keysym.sym == SDLK_DOWN)
        {
            my_key_event = DOWN_KEY_RELEASED;
        }
        else if (event->key.keysym.sym == SDLK_LEFT)
        {
            my_key_event = LEFT_KEY_RELEASED;
        } // if ... else
    } // if ... else
         
} // FS::GameManager::analyse_event() 

// **************************************************

void FS::GameManager::handle_selection(TitleScreenDisplay::Selection sel)
{
    switch (sel)
    {
        case TitleScreenDisplay::NONE:
            break;
    	case TitleScreenDisplay::PLAY:
            current_state = GET_READY;
            p_model.new_game();
            gr_display.start_draw( timer.get_time(), p_model.get_level() );
            break;
        case TitleScreenDisplay::SETTINGS:
            cerr << "settings\n";
            break;
        case TitleScreenDisplay::HI_SCORES:
            cerr << "hi-scores\n";
            break;
        case TitleScreenDisplay::QUIT:
            //exit(EXIT_SUCCESS);
            loop_running = false;
            break;
    } // switch
    
} // FS::GameManager::handle_selection()
 
// **************************************************
// **************************************************
// **************************************************
// **************************************************
// **************************************************
