//  Fish Supper
//  Copyright (C) 2006 Matthew Clarke <matthew_larcke@lycos.co.uk>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.




#include "SDL.h"
#include "SDL_mixer.h"
#include <iostream>
#include <cstdlib>
#include "enums.h"
#include "gamemanager.h"




using std::cout;
using std::cerr;
using std::endl;




int main(int argc, char * argv[])
{
    // This ensures that everything is cleaned up on exit.
    // N.B. Functions are called in reverse order.
    atexit(SDL_Quit);
    atexit(Mix_CloseAudio);
    
    // initialize SDL - need video, audio and joystick subsystems
    if ( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO|SDL_INIT_JOYSTICK) < 0 )
    {
        cerr << "Can't initialize SDL: " << SDL_GetError() << endl;
        exit(EXIT_FAILURE);
    } // if
    
    // set up audio (via SDL mixer)
    if ( Mix_OpenAudio(22050, AUDIO_S16SYS, 2, 1024) < 0 )
    {
        cerr << "Can't open audio: " << Mix_GetError() << endl;
        exit(EXIT_FAILURE);
    } // if
    
    // Get an SDL_Surface and pass control to GameManager
    SDL_Surface * screen = NULL;
    if ( (screen = SDL_SetVideoMode(FS::SCREEN_WIDTH, FS::SCREEN_HEIGHT, FS::BPP, 
            SDL_SWSURFACE|SDL_ANYFORMAT)) == NULL )
    {
        cerr << "Can't get an SDL surface: " << SDL_GetError() << endl;
        exit(EXIT_FAILURE);
    } // if
    
    // set window/icon title
    SDL_WM_SetCaption("Fish Supper - 0.1 (pre-alpha)", "Fish Supper");
     
    //SDL_Delay(5000);    // need SDL_INIT_TIMER for this function?
    FS::GameManager gm(screen);
    gm.start_loop();
    
    Mix_CloseAudio();
    SDL_Quit();
    
    return 0;
    
} // main()
