//  Fish Supper
//  Copyright (C) 2006 Matthew Clarke <matthew_larcke@lycos.co.uk>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.




#include "mapengine.h"
#include <iostream>

using std::cout;




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::MapEngine::MapEngine()
{
    my_blocks_allocated = false;
    
} // FS::MapEngine::MapEngine()

// ******************
// *** DESTRUCTOR ***
// ******************
FS::MapEngine::~MapEngine()
{
    if (my_blocks_allocated)
    {
        for (int i = 0; i < NUM_ROWS; ++i)
        {
            delete [] my_blocks[i];
        } // for
    } // if
    
} // FS::MapEngine::~MapEngine()




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************

void FS::MapEngine::load_map(ifstream & fin)
{
    int temp_int;
    float temp_float;
    
    // 1 row at a time: direction, velocity, row length, data
    for (int r = 0; r < NUM_ROWS; ++r)
    {
        int len;    // use this to control inner for loop
        
        // direction
        fin.read((char *) &temp_int, sizeof temp_int);
        directions[r] = (Direction) temp_int;
        
        // velocity
        fin.read((char *) &temp_float, sizeof temp_float);
        velocities[r] = temp_float;
        
        // row length 
        fin.read((char *) &len, sizeof len);
        row_lengths[r] = len;
        row_lengths_pixels[r] = len * BLOCK_WIDTH;
        
        // do we need to delete [] first?
        if (my_blocks_allocated)
        {
            delete [] my_blocks[r];
        } // if
        my_blocks[r] = new Block[len];
        
        for (int c = 0; c < len; ++c)   // 'c' for column
        {
            // log
            fin.read((char *) &temp_int, sizeof temp_int);
            my_blocks[r][c].lc = (LogColor) temp_int;
            // crystal
            fin.read((char *) &temp_int, sizeof temp_int);
            my_blocks[r][c].cc = (CrystalColor) temp_int;
        } // for c
    } // for r
    
    // set this to make sure memory cleared next time before loading
    // new data
    my_blocks_allocated = true;
    
} // FS::MapEngine::load_map()

// **************************************************

void FS::MapEngine::update(int time_passed)
{
    int temp_current_start_pixel;
    
    for (int r = 0; r < NUM_ROWS; ++r)
    {
        old_pixels_to_move[r] = pixels_to_move[r];
        pixels_to_move[r] = (int) (velocities[r] * time_passed);
        
        switch(directions[r])
        {
            case LEFT_TO_RIGHT:
                temp_current_start_pixel = pixels_to_move[r] % row_lengths_pixels[r];
                if (temp_current_start_pixel > 0)
                {
                    current_start_pixels[r] = 
                            row_lengths_pixels[r] - temp_current_start_pixel;                    
                }
                else
                {
                    current_start_pixels[r] = temp_current_start_pixel; // i.e. 0
                } // if ... else
                break;
                
            case RIGHT_TO_LEFT:
                current_start_pixels[r] = pixels_to_move[r] % row_lengths_pixels[r];
                break;
        } // switch
                        
        //current_end_pixels[r] = (current_start_pixels[r] + SCREEN_WIDTH) %
        //        row_lengths_pixels[r];
    } // for
    
} // FS::MapEngine::update(int t)

// **************************************************

void FS::MapEngine::reset()
{
    for (int i = 0; i < NUM_ROWS; ++i)
    {
        old_pixels_to_move[i] = 0;
    } // for
    
} // FS::MapEngine::reset()

// **************************************************
