//  Fish Supper
//  Copyright (C) 2006 Matthew Clarke <matthew_larcke@lycos.co.uk>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.




#ifndef _MAP_ENGINE_H_
#define _MAP_ENGINE_H_




#include "constants.h"
#include "enums.h"
#include <fstream>

using std::ifstream;




namespace FS
{
    
class MapEngine
{
    private:
            
        // 2D array of blocks and supporting data   
        Block * my_blocks[NUM_ROWS];
        int row_lengths[NUM_ROWS];
        int row_lengths_pixels[NUM_ROWS];
        float velocities[NUM_ROWS];
        Direction directions[NUM_ROWS];

        int current_start_pixels[NUM_ROWS];
        int current_end_pixels[NUM_ROWS];   // are these ever used?
        
        int pixels_to_move[NUM_ROWS];
        int old_pixels_to_move[NUM_ROWS];
        
        bool my_blocks_allocated;

    public:
            
        MapEngine();
        ~MapEngine();
        
        void load_map(ifstream & fin);
        
        void reset();
        
        void update(int t);
        
        const Block * const * get_blocks() { return my_blocks; }
        const int * get_row_lengths() { return row_lengths; }
        const int * get_row_lengths_pixels() { return row_lengths_pixels; }
        const Direction * get_directions() { return directions; }
        
        const int * get_current_start_pixels() { return current_start_pixels; }
        const int * get_pixels_to_move() { return pixels_to_move; }
        const int * get_old_pixels_to_move() { return old_pixels_to_move; }
                 
}; // class MapEngine

} // namespace FS




#endif
