//  Fish Supper
//  Copyright (C) 2006 Matthew Clarke <matthew_larcke@lycos.co.uk>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.




#include "playmodel.h"

using std::ifstream;
using std::cerr;
using std::cout;
using std::endl;




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::PlayModel::PlayModel()
{
    // ...
    
} // FS::PlayModel::PlayModel()

// ******************
// *** DESTRUCTOR ***
// ******************
FS::PlayModel::~PlayModel()
{
    // ...
    
} // FS::PlayModel::~PlayModel()




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************

void FS::PlayModel::load_level(int lev)
{
    // construct filename from 'lev'
    char my_filename[18];
    sprintf(my_filename, "data/levels/level_%02d.fsl", lev);
    //sprintf(my_filename, "data/levels/test.fsl");
    
    // open file - if can't open it that's a v. serious error so exit
    // program with message
    ifstream fin(my_filename);
    if (!fin.is_open())
    {
        cerr << "Can't open " << my_filename << ".\n";
        exit(EXIT_FAILURE);
    } // if
    
    // Used to hold values as read from file...
    int temp_int;
    
    // max crystals
    fin.read((char *) &temp_int, sizeof temp_int);
    my_player_data.set_max_crystals(temp_int);
    
    // number of starting crystals
    fin.read((char *) &num_starting_crystals, sizeof num_starting_crystals);
    
    // the starting crystals themselves - stored in this class 
    // in case we need to restart the level
    for (int i = 0; i < num_starting_crystals; ++i)
    {
        fin.read((char *) &starting_crystals[i], sizeof starting_crystals[i]);
    } // for
    my_player_data.set_crystals_held(starting_crystals, num_starting_crystals); 

    // number of crystals on map
    fin.read((char *) &temp_int, sizeof temp_int);
    my_sprite_mngr.set_num_crystals(temp_int);
    
    // number of logs per row
    for (int i = 0; i < NUM_ROWS; ++i)
    {
        fin.read((char *) &temp_int, sizeof temp_int);
        my_sprite_mngr.set_num_logs(temp_int, i);
    } // for
    
    // Hand stream over to MapEngine to let it initialize itself.
    my_map_engine.load_map(fin);
        
    fin.close();
    
} // FS::PlayModel::load_level()

// **************************************************

/*
void FS::PlayModel::reset(ResetMessage msg, int l, int st)
{
    if (msg == NEW_GAME)
    {
        my_player_data.reset(2);
    } // if
    
    load_level(l);  // need access to MapEngine's private members
    
    // Call following on SpriteManager - needs values from MapEngine.
    my_sprite_mngr.create_logs(my_map_engine.get_row_lengths(), my_map_engine.get_blocks(),
            my_map_engine.get_row_lengths_pixels(), my_map_engine.get_directions());
    my_sprite_mngr.create_crystals(my_map_engine.get_row_lengths(), 
            my_map_engine.get_blocks(), my_map_engine.get_row_lengths_pixels(), 
            my_map_engine.get_directions());
    
    my_sprite_mngr.reset();
    my_map_engine.reset();
    
    start_time = st;
    
    // TEMP!!!
    //time_allowed = time_left = 80000;

} // FS::PlayModel::reset()
*/
        
// **************************************************

FS::PlayReport FS::PlayModel::update(int gt, KeyboardEvent ke)
{
    if (ke == ESCAPE_PRESSED)
    {
        // FIXME - hack!
        my_sprite_mngr.stop_sound();
        return PR_GAME_ABORTED;
    } // if
    
    my_map_engine.update(gt - start_time);
    
    PlayReport my_report;
    
    switch ( my_sprite_mngr.update(gt - start_time, ke, my_map_engine.get_current_start_pixels(),
            my_map_engine.get_pixels_to_move(), my_map_engine.get_old_pixels_to_move()) )
    {
        case SpriteManager::PLAYING_GAME:
            my_report = PR_OK;
            break;
        case SpriteManager::REQUEST_LEVEL_RESTART:
            my_player_data.set_crystals_held(starting_crystals, num_starting_crystals);
            my_sprite_mngr.restart_level();
            my_report = PR_OK;
            break;
        case SpriteManager::LEVEL_COMPLETED:
            my_player_data.inc_level();
            my_report = PR_LEVEL_COMPLETED;
            break;
        case SpriteManager::GAME_OVER:
            my_report = PR_GAME_OVER;
            break;
        case SpriteManager::RUNNING_LEVEL_COMPLETED_ROUTINE:
        case SpriteManager::RUNNING_GAME_OVER_ROUTINE:
            my_report = PR_OK;
            break;
    } // switch
    
    return my_report;
    
} // FS::PlayModel::update()

// **************************************************

void FS::PlayModel::init_sprite_manager(PlayDisplay * pd, SoundEngine * se)
{
    my_sprite_mngr.set_play_display(pd);
    my_sprite_mngr.set_player_data(&my_player_data);
    my_sprite_mngr.set_sound_engine(se);
    
} // FS::PlayModel::init_sprite_manager()
      
// **************************************************

void FS::PlayModel::new_game()
{
    my_player_data.reset();
    
} // FS::PlayModel::new_game()
     
// **************************************************

void FS::PlayModel::new_level(int st)
{
    load_level(my_player_data.get_level());  // need access to MapEngine's private members
    
    // Call following on SpriteManager - needs values from MapEngine.
    my_sprite_mngr.create_logs(my_map_engine.get_row_lengths(), my_map_engine.get_blocks(),
            my_map_engine.get_row_lengths_pixels(), my_map_engine.get_directions());
    my_sprite_mngr.create_crystals(my_map_engine.get_row_lengths(), 
            my_map_engine.get_blocks(), my_map_engine.get_row_lengths_pixels(), 
            my_map_engine.get_directions());
    
    my_sprite_mngr.reset();
    my_map_engine.reset();
    
    start_time = st;
    
} // FS::PlayModel::new_level()

// **************************************************
// **************************************************
// **************************************************
// **************************************************
