//  Fish Supper
//  Copyright (C) 2006 Matthew Clarke <matthew_larcke@lycos.co.uk>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.




#include "titlescreendisplay.h"
#include "SDL_image.h"
#include <iostream>
#include <cstdlib>




using std::cerr;
using std::endl;




// *******************
// *** CONSTRUCTOR ***
// *******************
FS::TitleScreenDisplay::TitleScreenDisplay(SDL_Surface * s)
{
    // save a pointer to main screen for drawing on
    screen = s;
    
    // load the images we need
    backdrop = IMG_Load("data/images/intro_screen.png");    
    
    highlighted_options[0] = SDL_DisplayFormat(IMG_Load("data/images/play_h.png"));
    highlighted_options[1] = SDL_DisplayFormat(IMG_Load("data/images/settings_h.png"));
    highlighted_options[2] = SDL_DisplayFormat(IMG_Load("data/images/hiscores_h.png"));
    highlighted_options[3] = SDL_DisplayFormat(IMG_Load("data/images/quit_h.png"));
    
    // set up transparency (= color key) for highlighted images
    Uint32 transparency = SDL_MapRGB(screen->format, 30, 100, 200);
    for (int i = 0; i < NUM_OPTIONS; ++i)
    {
        SDL_SetColorKey(highlighted_options[i], SDL_SRCCOLORKEY|SDL_RLEACCEL, transparency);
    } // for
    
    if (backdrop == NULL)
    {
        cerr << "Can't load image: " << SDL_GetError() << endl;
        exit(EXIT_FAILURE);
    } // if

    for (int i = 0; i < 2; ++i)
    {
        dirty_rects[i].x = OPTIONS_X;
        dirty_rects[i].w = OPTIONS_WIDTH;
        dirty_rects[i].h = OPTIONS_HEIGHT;
    } // for
       
    int temp[] = { 180, 250, 320, 390 };
    for (int i = 0; i < NUM_OPTIONS; ++i)
    {
        options_ys[i] = temp[i];
    } // for
     
    current_selection = 0;
  
} // FS::TitleScreenDisplay::TitleScreenDisplay()    
 
// ******************
// *** DESTRUCTOR ***
// ******************
FS::TitleScreenDisplay::~TitleScreenDisplay()
{
    SDL_FreeSurface(backdrop);
    for (int i = 0; i < NUM_OPTIONS; ++i)
    {
        SDL_FreeSurface(highlighted_options[i]);
    } // for
    
} // FS::TitleScreenDisplay::~TitleScreenDisplay()    




// ************************
// *** MEMBER FUNCTIONS ***
// ************************

// **************************************************

void FS::TitleScreenDisplay::redraw()
{
    SDL_BlitSurface(backdrop, NULL, screen, NULL);
    
    SDL_Rect selection_dest = { OPTIONS_X, options_ys[current_selection] };
    SDL_BlitSurface(highlighted_options[current_selection], NULL,
            screen, &selection_dest);
            
    SDL_UpdateRect(screen, 0, 0, 0, 0); // TSD class responsible for this sort of operation?
    
} // FS::TitleScreenDisplay::redraw()

// **************************************************

void FS::TitleScreenDisplay::handle_keydown(SDLKey key)
{
    if ((key == SDLK_UP) && (current_selection > 0))
    {
        old_selection = current_selection;
        --current_selection;
        redraw_options();
    } 
    else if ((key == SDLK_DOWN) && (current_selection < (NUM_OPTIONS - 1)))
    {
        old_selection = current_selection;
        ++current_selection;
        redraw_options();
    } // if ... else
    
} // FS:TitleScreenDisplay::handle_keydown()

// **************************************************

// Returns true if user has pressed 'JUMP' key,
// else false (indicating no selection at present).
FS::TitleScreenDisplay::Selection FS::TitleScreenDisplay::update(KeyboardEvent ke)
{
    if ((ke == UP_KEY_PRESSED) && (current_selection > 0))
    {
    	old_selection = current_selection;
	--current_selection;
	redraw_options();
    }
    else if ((ke == DOWN_KEY_PRESSED) && (current_selection < (NUM_OPTIONS - 1)))
    {
    	old_selection = current_selection;
	++current_selection;
	redraw_options();
    }
    else if (ke == JUMP_KEY_PRESSED)
    {
    	return ((Selection) current_selection);
    } // if ... else
    
    return NONE;
 
} // FS::TitleScreenDisplay::update() 

// **************************************************

void FS::TitleScreenDisplay::redraw_options()
{    
    // repair background
    dirty_rects[0].y = options_ys[old_selection];
    SDL_BlitSurface(backdrop, &dirty_rects[0], screen, &dirty_rects[0]);
    
    // draw new highlighted selection
    dirty_rects[1].y = options_ys[current_selection];
    SDL_BlitSurface(highlighted_options[current_selection], NULL,
            screen, &dirty_rects[1]);
    
    SDL_UpdateRects(screen, 2, dirty_rects);
          
} // FS::TitleScreenDisplay::redraw_options()                
    
