//  Fish Supper
//  Copyright (C) 2006 Matthew Clarke <matthew_larcke@lycos.co.uk>
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
//  02111-1307, USA.




#ifndef _TITLE_SCREEN_DISPLAY_H_
#define _TITLE_SCREEN_DISPLAY_H_




#include "SDL.h"
#include "enums.h"




namespace FS
{
    
class TitleScreenDisplay
{
    public:
        enum Selection { PLAY, SETTINGS, HI_SCORES, QUIT, NONE };
    
    private:
        static const int NUM_OPTIONS = 4;
        // dimensions for ONE line...
        static const int OPTIONS_WIDTH = 285;
        static const int OPTIONS_HEIGHT = 65;
        static const int OPTIONS_X = 490;
        
        int options_ys[NUM_OPTIONS];
        SDL_Surface * highlighted_options[NUM_OPTIONS];
        SDL_Rect dirty_rects[2];    // for new and old selections
        
        int current_selection;
        int old_selection;
        
        SDL_Surface * backdrop;
        SDL_Surface * screen;   // from GameManager

        void redraw_options();
        
    public:
        TitleScreenDisplay(SDL_Surface * s);
        ~TitleScreenDisplay();  // use this to free SDL surfaces?
        
        void redraw();  // in future a base class in Display?
        Selection update(KeyboardEvent ke);    
        
        int get_current_selection() { return current_selection; }
        
        void handle_keydown(SDLKey key);
        void handle_keyup(SDLKey key);
        
}; // class TitleScreenDisplay
        
} // namespace FS




#endif
     
